@@ -21,8 +21,8 @@ class Background:
                 ):
         # load instead of instantiate
         if loadinfo:
-            self.loadinfo(savefolder)
-            return
+            self.background = self.loadinfo(savefolder)
+            return 
         # remove/create savefolder
         if overwrite:
             os.system('rm -r {0}'.format(savefolder))
@@ -48,7 +48,15 @@ class Background:
                      'tfile':tfile,
                      'extnum':extnum,
                      'ROOTNAME':'None',
-                     'savefolder':savefolder
+                     'savefolder':savefolder,
+                     'automask':automask,
+                     'rescale':rescale[0],
+                     'rescale_x1':rescale[1],
+                     'rescale_x2':rescale[2],
+                     'rescale_y':rescale[3],
+                     'sigclip':sigclip[0],
+                     'sigclip_niter':sigclip[1],
+                     'sigclip_sigma':sigclip[2]
                     }
         self.data['ROOTNAME'] = self._rootname()
         self.trace = self._trace()
@@ -58,13 +66,10 @@ class Background:
                      'bbox':None
                      }
         self.bbox['bbox'] = self._bbox()
-        self.automask = automask
-        self.rescale = rescale
-        self.sigclip = sigclip
         self.bkg = self._bkg()
     def _bkg(self):
-        rescale = self.rescale
-        sigclip = self.sigclip
+        rescale = (self.data['rescale'],self.data['rescale_x1'],self.data['rescale_x2'],self.data['rescale_y'])
+        sigclip = (self.data['sigclip'],self.data['sigclip_niter'],self.data['sigclip_sigma'])
         tmpdata = fits.open(self.data['gfile'])[self.data['extnum']].data
         bb0x,bb1x = self.bbox['bbox']['BBX']
         bb0y,bb1y = self.bbox['bbox']['BBY']
@@ -78,7 +83,7 @@ class Background:
         obj.data['MASK'] = self._mask(obj)
         return obj
     def _mask(self,obj):
-        if self.automask:
+        if self.data['automask']:
             tmpmask = np.full_like(obj.data['Y'],False,dtype=bool)
             halfdyup,halfdylow = self.bbox['halfdyup'],self.bbox['halfdylow']
             xgn = (self.bbox['bbox']['XG'] - self.bbox['bbox']['BBX'][0]).astype(int)
@@ -143,6 +148,35 @@ class Background:
         with open(filename) as tmp:
             tmpp = json.load(tmp)
         bbox = copy.deepcopy(tmpp)
+        # construct arguments
+        key_data = ['objname','gfile','tfile','extnum','savefolder','overwrite','loadinfo',
+               'automask','rescale','sigclip']
+        key_bbox = ['padyup','padylow','halfdyup','halfdylow','padxleft','padxright']
+        args = {}
+        for i in key_data:
+            if i=='savefolder':
+                args[i] = './tmp_loading_session'
+                continue
+            if i=='overwrite':
+                args[i] = True
+                continue
+            if i=='loadinfo':
+                args[i] = False
+                continue
+            if i=='rescale':
+                args[i] = (data['rescale'],data['rescale_x1'],data['rescale_x2'],data['rescale_y'])
+                continue
+            if i=='sigclip':
+                args[i] = (data['sigclip'],data['sigclip_niter'],data['sigclip_sigma'])
+                continue
+            args[i] = data[i]
+        for i in key_bbox:
+            args[i] = bbox[i]
+        # instantiate Background()
+        obj = Background(**args)
+        # fix savefolder
+        obj.data['savefolder'] = savefolder
+        # fix .bkg.model
         # load _coef.csv
         filename = glob.glob('./{0}/*_coef.csv'.format(savefolder))[0]
         tmp = pd.read_csv(filename,index_col=0)
@@ -161,44 +195,18 @@ class Background:
         filename = glob.glob('./{0}/*_pcov.csv'.format(savefolder))[0]
         tmp = pd.read_csv(filename,index_col=0)
         pcov = tmp.values.copy()
-        # construct
-        objname = data['objname']
-        gfile = data['gfile']
-        tfile = data['tfile']
-        extnum = data['extnum']
-        savefolder = savefolder
-        overwrite = False
-        loadinfo = False
-        padxleft = bbox['padxleft']
-        padxright = bbox['padxright']
-        padyup = bbox['padyup']
-        halfdyup = bbox['halfdyup']
-        halfdylow = bbox['halfdylow']
-        padylow = bbox['padylow']
-        adjustx = bbox['adjustx']
-        adjusty = bbox['adjusty']
-        obj = Background(objname=objname,gfile=gfile,tfile=tfile,extnum=extnum,
-                         savefolder=savefolder,overwrite=overwrite,loadinfo=loadinfo,
-                         padxleft=padxleft,padxright=padxright,
-                         padyup=padyup,halfdyup=halfdyup,halfdylow=halfdylow,padylow=padylow,
-                         adjustx=adjustx,adjusty=adjusty
-                        )
-        self.data = obj.data
-        self.trace = obj.trace
-        self.bbox = obj.bbox
-        self.bkg = Polynomial2D()
-        self.bkg.data = obj.bkg.data
-        self.bkg.model['NORDER'] = norder
-        self.bkg.model['COEF'] = coef
-        self.bkg.model['MASKFIT'] = maskfit
-        self.bkg.model['pcov'] = pcov
-        self.bkg.compute()
+        obj.bkg.model['NORDER'] = norder
+        obj.bkg.model['COEF'] = coef
+        obj.bkg.model['MASKFIT'] = maskfit
+        obj.bkg.model['pcov'] = pcov
+        obj.bkg.compute()
+        return(obj)
     ##########
     ##########
     ##########
     def saveinfo(self):
         savefolder = self.data['savefolder']
-        # save bkg sub
+        # save bkg sub as the same original name
         string = './{1}/{0}'.format(self.data['gfile'].split('/')[-1],savefolder)
         os.system('cp {0} ./{1}/'.format(self.data['gfile'],savefolder))
         tmpp = fits.open(string)
@@ -224,7 +232,7 @@ class Background:
         f.write(tmppp)
         f.close()
         print('Save {0}'.format(string))
-        # save only bkg
+        # save only bkg as *_bkg.fits
         string = './{1}/{0}'.format(self.data['gfile'].split('/')[-1].split('_')[0]+'_bkg.fits',savefolder)
         os.system('cp {0} {1}'.format(self.data['gfile'],string))
         tmpp = fits.open(string)
@@ -235,15 +243,6 @@ class Background:
         tmppdata[bb0y:bb1y,bb0x:bb1x] = yfit.copy()
         tmpp.writeto(string,overwrite=True)
         print('Save {0}'.format(string))
-        # save data
-        tmp = copy.deepcopy(self.data)
-        tmp['tfile'] = './{0}/{1}_trace.csv'.format(savefolder,self.data['ROOTNAME'])
-        tmppp = json.dumps(tmp)
-        string = './{1}/{0}_data.json'.format(self.data['ROOTNAME'],savefolder)
-        f = open(string,'w')
-        f.write(tmppp)
-        f.close()
-        print('Save {0}'.format(string))
         # save trace
         ##### xyref
         xref = np.full_like(self.trace['XG'],None,dtype=float)
@@ -259,9 +258,19 @@ class Background:
         tmp = {'XREF':xref,'YREF':yref,'XG':self.trace['XG'],'YG':self.trace['YG'],'WW':self.trace['WW'],
                'DYDX':dydx,'DLDP':dldp
               }
-        string = './{1}/{0}_trace.csv'.format(self.data['ROOTNAME'],savefolder)
-        pd.DataFrame(tmp).to_csv(string,index=False)
-        print('Save {0}'.format(string)) 
+        string_trace = './{1}/{0}_trace.csv'.format(self.data['ROOTNAME'],savefolder)
+        pd.DataFrame(tmp).to_csv(string_trace,index=False)
+        print('Save {0}'.format(string_trace)) 
+        # save data
+        # with new trace location
+        tmp = copy.deepcopy(self.data)
+        tmp['tfile'] = string_trace
+        tmppp = json.dumps(tmp)
+        string = './{1}/{0}_data.json'.format(self.data['ROOTNAME'],savefolder)
+        f = open(string,'w')
+        f.write(tmppp)
+        f.close()
+        print('Save {0}'.format(string))
         # save coef
         coef = self.bkg.model['COEF']
         norder = self.bkg.model['NORDER']
@@ -325,6 +334,7 @@ class Background:
         tmpx = np.arange(len(tmppp))
         ax2.plot(tmppp,tmpx)
         ax2.plot(np.full_like(tmppp,0.,dtype=float),tmpx,':')
+        ax2.set_xlabel('cps',fontsize=fontsize)
         ax = fig.add_subplot(2,2,3,sharex=ax)
         ax.plot(tmpp)
         tmpx = np.arange(tmpp.shape[0])
