@@ -9,6 +9,7 @@ import copy
 from .confreader import ConfReader
 from .computesip import ComputeSIP
 from .computetracenwavelength import ComputeTraceNWavelength
+from .flatfield import FlatField
 
 class Wfc3IrG102:
     def __init__(self,objname='None',dfile=None,gfile=None,extnum=4,
@@ -35,6 +36,8 @@ class Wfc3IrG102:
                       'YG':None,
                       'WW':None
                      }
+        self.flatfield = None
+        self.background = None
         self._get_root()
         self._get_nax()
         self._get_conf()
@@ -63,6 +66,17 @@ class Wfc3IrG102:
     ##########
     ##########
     ##########
+    def compute_flatfield(self,tracefile=None):
+        flatfield = self.data['flatfile']
+        rootname = self.data['ROOTNAME']
+        if tracefile is None:
+            tracefile = '{0}_trace.csv'.format(rootname)
+        obj = FlatField(flatfile=flatfile,tracefile=tracefile)
+        obj.compute()
+        self.flatfield = obj
+    ##########
+    ##########
+    ##########
     def compute(self):
         xyd = self.data['xyd']
         xydiff = self._compute_xydiff()
@@ -98,7 +112,8 @@ class Wfc3IrG102:
     ##########
     ##########
     ##########
-    def save(self,savename='trace.csv'):
+    def save(self,savename_prefix=None):
+        rootname = self.data['ROOTNAME']
         ##### xyref
         xyref = self.trace['XYREF']
         xref = np.full_like(self.trace['XG'],None,dtype=float)
@@ -119,13 +134,16 @@ class Wfc3IrG102:
         tmp = {'XREF':xref,'YREF':yref,'XG':self.trace['XG'],'YG':self.trace['YG'],'WW':self.trace['WW'],
                'DYDX':dydx,'DLDP':dldp
               }
-        pd.DataFrame(tmp).to_csv(savename,index=False)
-        print('Save {0}'.format(savename))
+        if savename_prefix is None:
+            savename_prefix = rootname
+        string = '{0}_trace.csv'.format(savename_prefix)
+        pd.DataFrame(tmp).to_csv(string,index=False)
+        print('Save {0}'.format(string))
     ##########
     ##########
     ##########
-    def show(self,save=False,savename='plot.eps',saveformat='eps',
-             dminmax=(5.,99.),gminmax=(5.,80.),alpha=0.6,lw=4,tickperx=50,
+    def show(self,save=False,savename_prefix=None,saveformat='pdf',
+             dminmax=(5.,99.),gminmax=(5.,80.),alpha=0.6,lw=4,xpertick=50,
              dxy=(50,50),rotation=30,
              figsize=(10,10),fontsize=12
             ):
@@ -183,7 +201,7 @@ class Wfc3IrG102:
         ax4.imshow(tmppdata,origin='lower',cmap='viridis',vmin=vmin,vmax=vmax)
         ax4.plot(xg,yg,'r:',alpha=alpha,lw=lw)
         for i,ii in enumerate(xg):
-            if (i in {0,len(xg)-1}) or (np.mod(i,tickperx)==0):
+            if (i in {0,len(xg)-1}) or (np.mod(i,xpertick)==0):
                 label = '{0}A'.format(int(ww[i]))
                 ax4.plot(xg[i],yg[i],'ro')
                 ax4.annotate(label,(xg[i],yg[i]),
@@ -200,6 +218,9 @@ class Wfc3IrG102:
         
         plt.tight_layout()
         if save:
-            plt.savefig(savename,format=saveformat,bbox_inches='tight')
-            print('Save {0}\n'.format(savename))
-        
+            if savename_prefix is None:
+                savename_prefix = root
+            string = '{0}_overview.{1}'.format(savename_prefix,saveformat)
+            plt.savefig(string,format=saveformat,bbox_inches='tight')
+            print('Save {0}\n'.format(string))
+        
\ No newline at end of file
